@@ -16,15 +16,15 @@ package com.google.googlejavaformat.java;
 
 import static com.google.common.collect.ImmutableList.toImmutableList;
 import static com.google.common.collect.Iterables.getLast;
+import static com.google.googlejavaformat.java.Trees.getEndPosition;
+import static com.google.googlejavaformat.java.Trees.getStartPosition;
 import static java.lang.Math.min;
-import static java.nio.charset.StandardCharsets.UTF_8;
 import static java.util.stream.Collectors.joining;
 
 import com.google.common.base.CharMatcher;
 import com.google.common.base.Strings;
 import com.google.common.base.Verify;
 import com.google.common.collect.ImmutableList;
-import com.google.common.collect.Iterables;
 import com.google.common.collect.Range;
 import com.google.common.collect.TreeRangeMap;
 import com.google.googlejavaformat.Newlines;
@@ -35,17 +35,9 @@ import com.sun.source.tree.Tree;
 import com.sun.source.tree.Tree.Kind;
 import com.sun.source.util.TreePath;
 import com.sun.source.util.TreePathScanner;
-import com.sun.tools.javac.file.JavacFileManager;
-import com.sun.tools.javac.parser.JavacParser;
-import com.sun.tools.javac.parser.ParserFactory;
 import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.util.Context;
-import com.sun.tools.javac.util.Log;
-import com.sun.tools.javac.util.Options;
 import com.sun.tools.javac.util.Position;
-import java.io.IOException;
-import java.io.UncheckedIOException;
-import java.net.URI;
 import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Deque;
@@ -54,11 +46,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicBoolean;
 import javax.tools.Diagnostic;
-import javax.tools.DiagnosticCollector;
-import javax.tools.DiagnosticListener;
 import javax.tools.JavaFileObject;
-import javax.tools.SimpleJavaFileObject;
-import javax.tools.StandardLocation;
 
 /** Wraps string literals that exceed the column limit. */
 public final class StringWrapper {
@@ -171,7 +159,7 @@ public final class StringWrapper {
             && ((MemberSelectTree) parent).getExpression().equals(literalTree)) {
           return null;
         }
-        int endPosition = getEndPosition(unit, literalTree);
+        int endPosition = getEndPosition(literalTree, unit);
         int lineEnd = endPosition;
         while (Newlines.hasNewlineAt(input, lineEnd) == -1) {
           lineEnd++;
@@ -188,7 +176,7 @@ public final class StringWrapper {
         TreeRangeMap<Integer, String> replacements, List<Tree> textBlocks) {
       for (Tree tree : textBlocks) {
         int startPosition = lineMap.getStartPosition(lineMap.getLineNumber(getStartPosition(tree)));
-        int endPosition = getEndPosition(unit, tree);
+        int endPosition = getEndPosition(tree, unit);
         String text = input.substring(startPosition, endPosition);
         int leadingWhitespace = CharMatcher.whitespace().negate().indexIn(text);
 
@@ -254,7 +242,7 @@ public final class StringWrapper {
 
         // Handling leaving trailing non-string tokens at the end of the literal,
         // e.g. the trailing `);` in `foo("...");`.
-        int end = getEndPosition(unit, getLast(flat));
+        int end = getEndPosition(getLast(flat), unit);
         int lineEnd = end;
         while (Newlines.hasNewlineAt(input, lineEnd) == -1) {
           lineEnd++;
@@ -264,7 +252,7 @@ public final class StringWrapper {
         // Get the original source text of the string literals, excluding `"` and `+`.
         ImmutableList<String> components = stringComponents(input, unit, flat);
         replacements.put(
-            Range.closedOpen(getStartPosition(flat.get(0)), getEndPosition(unit, getLast(flat))),
+            Range.closedOpen(getStartPosition(flat.get(0)), getEndPosition(getLast(flat), unit)),
             reflow(separator, columnLimit, startColumn, trailing, components, first.get()));
       }
     }
@@ -280,7 +268,7 @@ public final class StringWrapper {
     StringBuilder piece = new StringBuilder();
     for (Tree tree : flat) {
       // adjust for leading and trailing double quotes
-      String text = input.substring(getStartPosition(tree) + 1, getEndPosition(unit, tree) - 1);
+      String text = input.substring(getStartPosition(tree) + 1, getEndPosition(tree, unit) - 1);
       int start = 0;
       for (int idx = 0; idx < text.length(); idx++) {
         if (CharMatcher.whitespace().matches(text.charAt(idx))) {
@@ -453,20 +441,12 @@ public final class StringWrapper {
   private static boolean noComments(
       String input, JCTree.JCCompilationUnit unit, Tree one, Tree two) {
     return STRING_CONCAT_DELIMITER.matchesAllOf(
-        input.subSequence(getEndPosition(unit, one), getStartPosition(two)));
+        input.subSequence(getEndPosition(one, unit), getStartPosition(two)));
   }
 
   public static final CharMatcher STRING_CONCAT_DELIMITER =
       CharMatcher.whitespace().or(CharMatcher.anyOf("\"+"));
 
-  private static int getEndPosition(JCTree.JCCompilationUnit unit, Tree tree) {
-    return ((JCTree) tree).getEndPosition(unit.endPositions);
-  }
-
-  private static int getStartPosition(Tree tree) {
-    return ((JCTree) tree).getStartPosition();
-  }
-
   /**
    * Returns true if any lines in the given Java source exceed the column limit, or contain a {@code
    * """} that could indicate a text block.
@@ -486,34 +466,11 @@ public final class StringWrapper {
   /** Parses the given Java source. */
   private static JCTree.JCCompilationUnit parse(String source, boolean allowStringFolding)
       throws FormatterException {
-    DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<>();
+    List<Diagnostic<? extends JavaFileObject>> errorDiagnostics = new ArrayList<>();
     Context context = new Context();
-    context.put(DiagnosticListener.class, diagnostics);
-    Options.instance(context).put("--enable-preview", "true");
-    Options.instance(context).put("allowStringFolding", Boolean.toString(allowStringFolding));
-    JavacFileManager fileManager = new JavacFileManager(context, true, UTF_8);
-    try {
-      fileManager.setLocation(StandardLocation.PLATFORM_CLASS_PATH, ImmutableList.of());
-    } catch (IOException e) {
-      throw new UncheckedIOException(e);
-    }
-    SimpleJavaFileObject sjfo =
-        new SimpleJavaFileObject(URI.create("source"), JavaFileObject.Kind.SOURCE) {
-          @Override
-          public CharSequence getCharContent(boolean ignoreEncodingErrors) {
-            return source;
-          }
-        };
-    Log.instance(context).useSource(sjfo);
-    ParserFactory parserFactory = ParserFactory.instance(context);
-    JavacParser parser =
-        parserFactory.newParser(
-            source, /* keepDocComments= */ true, /* keepEndPos= */ true, /* keepLineMap= */ true);
-    JCTree.JCCompilationUnit unit = parser.parseCompilationUnit();
-    unit.sourcefile = sjfo;
-    Iterable<Diagnostic<? extends JavaFileObject>> errorDiagnostics =
-        Iterables.filter(diagnostics.getDiagnostics(), Formatter::errorDiagnostic);
-    if (!Iterables.isEmpty(errorDiagnostics)) {
+    JCTree.JCCompilationUnit unit =
+        Trees.parse(context, errorDiagnostics, allowStringFolding, source);
+    if (!errorDiagnostics.isEmpty()) {
       // error handling is done during formatting
       throw FormatterException.fromJavacDiagnostics(errorDiagnostics);
     }
